@@ -66,7 +66,7 @@ def defaults_file_preview() -> str:
 
 
 class TestDefaults:
-    def test_fix(self, hatch, temp_dir, config_file, env_run, mocker, platform, defaults_file_stable):
+    def test_fix(self, hatch, helpers, temp_dir, config_file, env_run, mocker, platform, defaults_file_stable):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -81,17 +81,21 @@ class TestDefaults:
         data_path = temp_dir / 'data'
         data_path.mkdir()
 
+        config_dir = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config' / project_path.id
+        default_config = config_dir / 'ruff_defaults.toml'
+        user_config = config_dir / 'pyproject.toml'
+        user_config_path = platform.join_command_args([str(user_config)])
+
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
-
-        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
-        config_dir = next(root_data_path.iterdir())
-        default_config = config_dir / 'ruff_defaults.toml'
-        user_config = config_dir / 'pyproject.toml'
-        user_config_path = platform.join_command_args([str(user_config)])
+        assert result.output == helpers.dedent(
+            f"""
+            cmd [1] | ruff check --config {user_config_path} --fix .
+            cmd [2] | ruff format --config {user_config_path} .
+            """
+        )
 
         assert env_run.call_args_list == [
             mocker.call(f'ruff check --config {user_config_path} --fix .', shell=True),
@@ -110,7 +114,7 @@ class TestDefaults:
 extend = "{config_path}\""""
         )
 
-    def test_check(self, hatch, temp_dir, config_file, env_run, mocker, platform, defaults_file_stable):
+    def test_check(self, hatch, helpers, temp_dir, config_file, env_run, mocker, platform, defaults_file_stable):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -125,17 +129,21 @@ extend = "{config_path}\""""
         data_path = temp_dir / 'data'
         data_path.mkdir()
 
+        config_dir = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config' / project_path.id
+        default_config = config_dir / 'ruff_defaults.toml'
+        user_config = config_dir / 'pyproject.toml'
+        user_config_path = platform.join_command_args([str(user_config)])
+
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--check')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
-
-        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
-        config_dir = next(root_data_path.iterdir())
-        default_config = config_dir / 'ruff_defaults.toml'
-        user_config = config_dir / 'pyproject.toml'
-        user_config_path = platform.join_command_args([str(user_config)])
+        assert result.output == helpers.dedent(
+            f"""
+            cmd [1] | ruff check --config {user_config_path} .
+            cmd [2] | ruff format --config {user_config_path} --check --diff .
+            """
+        )
 
         assert env_run.call_args_list == [
             mocker.call(f'ruff check --config {user_config_path} .', shell=True),
@@ -154,7 +162,9 @@ extend = "{config_path}\""""
 extend = "{config_path}\""""
         )
 
-    def test_existing_config(self, hatch, temp_dir, config_file, env_run, mocker, platform, defaults_file_stable):
+    def test_existing_config(
+        self, hatch, helpers, temp_dir, config_file, env_run, mocker, platform, defaults_file_stable
+    ):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -169,6 +179,11 @@ extend = "{config_path}\""""
         data_path = temp_dir / 'data'
         data_path.mkdir()
 
+        config_dir = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config' / project_path.id
+        default_config = config_dir / 'ruff_defaults.toml'
+        user_config = config_dir / 'pyproject.toml'
+        user_config_path = platform.join_command_args([str(user_config)])
+
         project_file = project_path / 'pyproject.toml'
         old_contents = project_file.read_text()
         project_file.write_text(f'[tool.ruff]\n{old_contents}')
@@ -177,13 +192,12 @@ extend = "{config_path}\""""
             result = hatch('fmt', '--check')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
-
-        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
-        config_dir = next(root_data_path.iterdir())
-        default_config = config_dir / 'ruff_defaults.toml'
-        user_config = config_dir / 'pyproject.toml'
-        user_config_path = platform.join_command_args([str(user_config)])
+        assert result.output == helpers.dedent(
+            f"""
+            cmd [1] | ruff check --config {user_config_path} .
+            cmd [2] | ruff format --config {user_config_path} --check --diff .
+            """
+        )
 
         assert env_run.call_args_list == [
             mocker.call(f'ruff check --config {user_config_path} .', shell=True),
@@ -203,7 +217,7 @@ extend = "{config_path}\"
 
 
 class TestPreview:
-    def test_fix_flag(self, hatch, temp_dir, config_file, env_run, mocker, platform, defaults_file_preview):
+    def test_fix_flag(self, hatch, helpers, temp_dir, config_file, env_run, mocker, platform, defaults_file_preview):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -218,17 +232,21 @@ class TestPreview:
         data_path = temp_dir / 'data'
         data_path.mkdir()
 
+        config_dir = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config' / project_path.id
+        default_config = config_dir / 'ruff_defaults.toml'
+        user_config = config_dir / 'pyproject.toml'
+        user_config_path = platform.join_command_args([str(user_config)])
+
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--preview')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
-
-        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
-        config_dir = next(root_data_path.iterdir())
-        default_config = config_dir / 'ruff_defaults.toml'
-        user_config = config_dir / 'pyproject.toml'
-        user_config_path = platform.join_command_args([str(user_config)])
+        assert result.output == helpers.dedent(
+            f"""
+            cmd [1] | ruff check --config {user_config_path} --preview --fix .
+            cmd [2] | ruff format --config {user_config_path} --preview .
+            """
+        )
 
         assert env_run.call_args_list == [
             mocker.call(f'ruff check --config {user_config_path} --preview --fix .', shell=True),
@@ -247,7 +265,7 @@ class TestPreview:
 extend = "{config_path}\""""
         )
 
-    def test_check_flag(self, hatch, temp_dir, config_file, env_run, mocker, platform, defaults_file_preview):
+    def test_check_flag(self, hatch, helpers, temp_dir, config_file, env_run, mocker, platform, defaults_file_preview):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -262,17 +280,21 @@ extend = "{config_path}\""""
         data_path = temp_dir / 'data'
         data_path.mkdir()
 
+        config_dir = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config' / project_path.id
+        default_config = config_dir / 'ruff_defaults.toml'
+        user_config = config_dir / 'pyproject.toml'
+        user_config_path = platform.join_command_args([str(user_config)])
+
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--check', '--preview')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
-
-        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
-        config_dir = next(root_data_path.iterdir())
-        default_config = config_dir / 'ruff_defaults.toml'
-        user_config = config_dir / 'pyproject.toml'
-        user_config_path = platform.join_command_args([str(user_config)])
+        assert result.output == helpers.dedent(
+            f"""
+            cmd [1] | ruff check --config {user_config_path} --preview .
+            cmd [2] | ruff format --config {user_config_path} --preview --check --diff .
+            """
+        )
 
         assert env_run.call_args_list == [
             mocker.call(f'ruff check --config {user_config_path} --preview .', shell=True),
@@ -407,7 +429,7 @@ extend = "{config_path}\""""
 
 
 class TestArguments:
-    def test_forwarding(self, hatch, temp_dir, config_file, env_run, mocker, platform, defaults_file_stable):
+    def test_forwarding(self, hatch, helpers, temp_dir, config_file, env_run, mocker, platform, defaults_file_stable):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -422,17 +444,21 @@ class TestArguments:
         data_path = temp_dir / 'data'
         data_path.mkdir()
 
+        config_dir = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config' / project_path.id
+        default_config = config_dir / 'ruff_defaults.toml'
+        user_config = config_dir / 'pyproject.toml'
+        user_config_path = platform.join_command_args([str(user_config)])
+
         with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
             result = hatch('fmt', '--', '--foo', 'bar')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
-
-        root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
-        config_dir = next(root_data_path.iterdir())
-        default_config = config_dir / 'ruff_defaults.toml'
-        user_config = config_dir / 'pyproject.toml'
-        user_config_path = platform.join_command_args([str(user_config)])
+        assert result.output == helpers.dedent(
+            f"""
+            cmd [1] | ruff check --config {user_config_path} --fix --foo bar
+            cmd [2] | ruff format --config {user_config_path} --foo bar
+            """
+        )
 
         assert env_run.call_args_list == [
             mocker.call(f'ruff check --config {user_config_path} --fix --foo bar', shell=True),
@@ -479,7 +505,7 @@ class TestConfigPath:
             """
         )
 
-    def test_sync(self, hatch, temp_dir, config_file, env_run, mocker, defaults_file_stable):
+    def test_sync(self, hatch, helpers, temp_dir, config_file, env_run, mocker, defaults_file_stable):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -506,7 +532,12 @@ class TestConfigPath:
             result = hatch('fmt', '--sync')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
+        assert result.output == helpers.dedent(
+            """
+            cmd [1] | ruff check --fix .
+            cmd [2] | ruff format .
+            """
+        )
 
         root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         assert not root_data_path.is_dir()
@@ -518,7 +549,7 @@ class TestConfigPath:
 
         assert default_config_file.read_text() == defaults_file_stable
 
-    def test_no_sync(self, hatch, temp_dir, config_file, env_run, mocker):
+    def test_no_sync(self, hatch, helpers, temp_dir, config_file, env_run, mocker):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -545,7 +576,12 @@ class TestConfigPath:
             result = hatch('fmt')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
+        assert result.output == helpers.dedent(
+            """
+            cmd [1] | ruff check --fix .
+            cmd [2] | ruff format .
+            """
+        )
 
         root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         assert not root_data_path.is_dir()
@@ -557,7 +593,7 @@ class TestConfigPath:
 
         assert not default_config_file.read_text()
 
-    def test_sync_legacy_config(self, hatch, temp_dir, config_file, env_run, mocker, defaults_file_stable):
+    def test_sync_legacy_config(self, hatch, helpers, temp_dir, config_file, env_run, mocker, defaults_file_stable):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -584,9 +620,12 @@ class TestConfigPath:
             result = hatch('fmt', '--sync')
 
         assert result.exit_code == 0, result.output
-        assert result.output == (
-            'The `tool.hatch.format.config-path` option is deprecated and will be removed in a future release. '
-            'Use `tool.hatch.envs.hatch-static-analysis.config-path` instead.\n'
+        assert result.output == helpers.dedent(
+            """
+            The `tool.hatch.format.config-path` option is deprecated and will be removed in a future release. Use `tool.hatch.envs.hatch-static-analysis.config-path` instead.
+            cmd [1] | ruff check --fix .
+            cmd [2] | ruff format .
+            """
         )
 
         root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
@@ -701,7 +740,7 @@ class TestCustomScripts:
             mocker.call('flake8 .', shell=True),
         ]
 
-    def test_only_formatter_fix(self, hatch, temp_dir, config_file, env_run, mocker):
+    def test_only_formatter_fix(self, hatch, helpers, temp_dir, config_file, env_run, mocker):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -742,7 +781,12 @@ class TestCustomScripts:
             result = hatch('fmt', '--formatter')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
+        assert result.output == helpers.dedent(
+            """
+            cmd [1] | isort .
+            cmd [2] | black .
+            """
+        )
 
         root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         assert not root_data_path.is_dir()
@@ -752,7 +796,7 @@ class TestCustomScripts:
             mocker.call('black .', shell=True),
         ]
 
-    def test_only_formatter_check(self, hatch, temp_dir, config_file, env_run, mocker):
+    def test_only_formatter_check(self, hatch, helpers, temp_dir, config_file, env_run, mocker):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -793,7 +837,12 @@ class TestCustomScripts:
             result = hatch('fmt', '--check', '--formatter')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
+        assert result.output == helpers.dedent(
+            """
+            cmd [1] | black --check --diff .
+            cmd [2] | isort --check-only --diff .
+            """
+        )
 
         root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         assert not root_data_path.is_dir()
@@ -803,7 +852,7 @@ class TestCustomScripts:
             mocker.call('isort --check-only --diff .', shell=True),
         ]
 
-    def test_fix(self, hatch, temp_dir, config_file, env_run, mocker):
+    def test_fix(self, hatch, helpers, temp_dir, config_file, env_run, mocker):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -844,7 +893,13 @@ class TestCustomScripts:
             result = hatch('fmt')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
+        assert result.output == helpers.dedent(
+            """
+            cmd [1] | flake8 .
+            cmd [2] | isort .
+            cmd [3] | black .
+            """
+        )
 
         root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         assert not root_data_path.is_dir()
@@ -855,7 +910,7 @@ class TestCustomScripts:
             mocker.call('black .', shell=True),
         ]
 
-    def test_check(self, hatch, temp_dir, config_file, env_run, mocker):
+    def test_check(self, hatch, helpers, temp_dir, config_file, env_run, mocker):
         config_file.model.template.plugins['default']['tests'] = False
         config_file.save()
 
@@ -896,7 +951,13 @@ class TestCustomScripts:
             result = hatch('fmt', '--check')
 
         assert result.exit_code == 0, result.output
-        assert not result.output
+        assert result.output == helpers.dedent(
+            """
+            cmd [1] | flake8 .
+            cmd [2] | black --check --diff .
+            cmd [3] | isort --check-only --diff .
+            """
+        )
 
         root_data_path = data_path / 'env' / '.internal' / 'hatch-static-analysis' / '.config'
         assert not root_data_path.is_dir()
